package main

import (
	"bufio"
	"errors"
	"fmt"
	"io"
	"os"
	"strconv"
	"strings"
)

func importFile(file string) ([][]int, error) {
	var err error
	var mySlice = make([][]int, 0)

	f, err := os.Open(file)
	if err != nil {
		return nil, err
	}
	defer f.Close()

	r := bufio.NewReader(f)
	for {
		line, err := r.ReadString('\n')
		fields := strings.Fields(line)
		temp := make([]int, 0)
		for _, v := range fields {
			n, err := strconv.Atoi(v)
			if err != nil {
				return nil, err
			}
			temp = append(temp, n)
		}
		if len(temp) != 0 {
			mySlice = append(mySlice, temp)
		}

		if err == io.EOF {
			break
		} else if err != nil {
			return nil, err
		}

		if len(temp) != len(mySlice[0]) {
			return nil, errors.New("Wrong number of elements!")
		}

	}
	return mySlice, nil
}

func validPuzzle(sl [][]int) bool {
	for i := 0; i <= 8; i++ {
		mySlice := []int{0, 0, 0, 0, 0, 0, 0, 0, 0}
		x := (i % 3) * 3
		y := (i / 3) * 3
		for j := 0; j <= 8; j++ {
			k := (j % 3)
			m := (j / 3)
			bigI := x + k
			bigJ := y + m
			val := sl[bigI][bigJ]
			if val > 0 && val < 10 {
				if mySlice[val-1] == 1 {
					fmt.Println("Value appeared two times!")
					return false
				} else {
					mySlice[val-1] = 1
				}
			} else {
				fmt.Println("Invalid value!")
				return false
			}
		}
	}

	// Testing columns + rows
	for i := 0; i <= 8; i++ {
		s1 := 0
		s2 := 0
		for j := 0; j <= 8; j++ {
			s1 = s1 + sl[j][i]
			s2 = s2 + sl[i][j]
		}
		if s1+s2 != 90 {
			return false
		}
		s1 = 0
		s2 = 0
	}
	return true
}

func main() {
	arguments := os.Args
	if len(arguments) == 1 {
		fmt.Printf("usage: loadFile textFile\n")
		return
	}

	file := arguments[1]
	mySlice, err := importFile(file)
	if err != nil {
		fmt.Println(err)
		return
	}

	if validPuzzle(mySlice) {
		fmt.Println("Correct Sudoku puzzle!")
	} else {
		fmt.Println("Incorrect Sudoku puzzle!")
	}
}
